

-------------------------------------------------------------------------------
-- vga.vhd
--
-- Author(s):     Ashley Partis and Jorgen Peddersen
-- Created:       Jan 2001
-- Last Modified: Jan 2001
-- 
-- This code acts as a top level for the VGA output project.  The RAMDAC should
-- be set up to program for pseudo colour.  This displays a pattern
-- of horizontal or vertical bands of every colour in the colour map.
-- Experimentation is the best way to understand what each switch does to
-- the displayed colour map.
-------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

entity vga is
    port (
        clk: in STD_LOGIC;								-- clock
        rstn: in STD_LOGIC;								-- asynchronous active low reset

        hit0: in STD_LOGIC_VECTOR (1 downto 0);	
		hit1: in STD_LOGIC_VECTOR (1 downto 0);
		hit2: in STD_LOGIC_VECTOR (1 downto 0);
		hit3: in STD_LOGIC_VECTOR (1 downto 0);
        wholelist : in  STD_LOGIC_VECTOR(119 downto 0);
        grade : IN STD_LOGIC_VECTOR(1 downto 0);
		sdone : IN STD_LOGIC;
        
		pixel: out STD_LOGIC_VECTOR (7 downto 0);		-- RAMDAC pixel lines
        blankn: out STD_LOGIC;							-- RAMDAC blank signal
        RDn: out STD_LOGIC;								-- RAMDAC RDn connection
		WRn: out STD_LOGIC;        						-- RAMDAC WRn connection
		RAMDACD: inout STD_LOGIC_VECTOR (7 downto 0);	-- RAMDAC data lines
		RS: inout STD_LOGIC_VECTOR (2 downto 0);		-- RAMDAC RS lines
        hsync: out STD_LOGIC;							-- horizontal sync for monitor
        vsync: out STD_LOGIC;							-- vertical sync for monitor
        triste: out STD_LOGIC;							-- signal to tristate ethernet PHY
        rramce: out STD_LOGIC;							-- right ram chip enable
        pixelclk: out STD_LOGIC							-- RAMDAC pixel clock
    );
end vga;

architecture vga_arch of vga is

-- control VGA signals
component vgacore 
	generic (
		H_SIZE : integer;							-- horizontal size of input image, MAX 800
		V_SIZE : integer							-- vertical size of input image, MAX 600
	);
	port
	(
		reset: in std_logic;						-- asynchronous active low reset
		clock: in std_logic;						-- clock
		hsyncb: buffer std_logic;					-- horizontal (line) sync
		vsyncb: out std_logic;						-- vertical (frame) sync
		latch: out STD_LOGIC;						-- latches new rgb value
		enable: out STD_LOGIC;						-- enable/ground RGB output lines
		hloc: out std_logic_vector(9 downto 0);		-- horizontal address to be decoded for video RAM
		vloc: out std_logic_vector(9 downto 0)		-- vertical address to be decoded for video RAM
	);
end component;

-- Program the RAMDAC
component prgramdacver2
    port (
        clk: in STD_LOGIC;							-- Clock
        rstn: in STD_LOGIC;							-- Asynchronous active low reset
        start: in STD_LOGIC;						-- Start signal
        done: out STD_LOGIC;						-- Asserted when programming is finished
        WRn: out STD_LOGIC;							-- Write line to RAMDAC (active low)
        RDn: out STD_LOGIC;							-- Read line to RAMDAC (active low)
        RS: inout STD_LOGIC_VECTOR (2 downto 0);	-- Register select lines to the RAMDAC
        data: inout STD_LOGIC_VECTOR (7 downto 0)	-- Bidirectional data line to RAMDAC
    );
end component;

-- State signals
type STATETYPE is (stReset, stWaste1, stWaste2, stWaste3, stWait, stForever);

signal presState: STATETYPE;
signal globalcount : integer range 0 to 501 := 0;

signal hsyncInt : STD_LOGIC;
signal vsyncInt : STD_LOGIC;

-- signals to cue different processes
signal startProg : STD_LOGIC;
signal startVGA : STD_LOGIC;
signal resetVGA : STD_LOGIC;
signal done : STD_LOGIC;

-- signals to generate test pattern
signal hloc : std_logic_vector(9 downto 0);				-- horizontal location of each pixel
signal vloc : std_logic_vector(9 downto 0);				-- vertical location of each pixel
signal colouroffset : std_logic_vector(7 downto 0);		-- offset for rotation
signal pixelData : STD_LOGIC_VECTOR(7 downto 0);		-- colour to write for each pixel

begin
	-- VGA controller
	cycler : vgacore 
	generic map (
		H_SIZE => 800,
		V_SIZE => 600
	)
	port map(
		reset => resetVGA,
		clock => clk,
		hsyncb => hsyncInt,
		vsyncb => vsyncInt,
		latch => open,
		enable => blankn,
		hloc => hloc,
		vloc => vloc
	);
    -- RAMDAC programmer    
    RAMDACprog : prgramdacver2 port map (
        clk => clk,
        rstn => rstn,
        start => startProg,
        done => done,
        WRn => WRn,
        RDn => RDn,
        RS => RS,
        data => RAMDACD
    );

	-- This is a simple mealy state machine that cues the VGA controller
	-- when the RAMDAC is finished programming
	process(clk, rstn)
	begin
		if rstn = '0' then
			presState <= stReset;
		elsif clk'event AND clk = '1' then
			case presState is
				when stReset =>
					presState <= stWaste1;
				when stWaste1 =>
					presState <= stWaste2;
				when stWaste2 =>
					presState <= stWaste3;
				when stWaste3 =>
					presState <= stWait;
				when stWait =>
					if done = '0' then
						presState <= stWait;
					else
						presState <= stForever;
					end if;
				when stForever =>
					presState <= stForever;
			end case;
		end if;
	end process;
	
	process(presState)
	begin
		case presState is
			when stReset =>
				startProg <= '1';
				startVGA <= '0';
			when stWaste1 =>
				startProg <= '1';
				startVGA <= '0';
			when stWaste2 =>
				startProg <= '1';
				startVGA <= '0';
			when stWaste3 =>
				startProg <= '1';
				startVGA <= '0';
			when stWait =>
				startProg <= '0';
				startVGA <= '0';
			when stForever =>
				startProg <= '0';
				startVGA <= '1';
		end case;
	end process;

	  
	-- calculate offset for data that is rotating.  Value will update whenever
	-- a new page is called for as it is clocked by vsync.

-----------------------------------------------------------------------------------------------------
-- Calculate pixelData.  This is the colour programmed into each pixel.
-- By choosing different combinations of the three component colours (red, green and blue)
-- different patterns can be formed.  For each colour, the following 
-- patterns can be chosen from:
process(clk)
	begin

      if (vsyncInt'event and vsyncInt = '1') then   
--prevent overflow:
           if (globalcount = 500) then 
				    globalcount <= 0;
		   else   
		          globalcount <= globalcount + 1;
		   end if;
      end if;
        
end process;
-------------------------------------------------------------------------------------------------------
process (vloc,hloc,sdone)

variable pd : STD_LOGIC_VECTOR(7 downto 0);
variable shine : STD_LOGIC_VECTOR(7 downto 0);
variable notbelowbasebar : boolean;
variable secondbar : boolean;
variable thirdbar : boolean;
variable widthofbar : boolean;
variable firstbar : boolean;
variable forthbar : boolean;
variable newvv : integer range 0 to 600; 
variable i : integer range 0 to 20;
variable j : integer range 0 to 200 := 0;
variable hitrange : boolean;

begin

shine := "00000000" + globalcount;
pd := "11111111";
newvv := 200;

--conditions
notbelowbasebar := vloc < 500;
secondbar := hloc >= 270 and hloc <= 370;
firstbar := hloc >= 110 and hloc <=210;
thirdbar := hloc >= 430 and hloc <= 530;
forthbar := hloc >= 590 and hloc <= 690;

--hitrange := (vloc >= 490) and notbelowbasebar;
hitrange := (vloc >= 200) and notbelowbasebar;

--background
pixelData <= "00000000";

--separator
if vloc > 200 and notbelowbasebar and ((hloc >=235 and hloc <= 245) or (hloc >= 385 and hloc <= 395) or (hloc >= 540 and hloc <= 550)) then
	pixelData <= "11111111";
--base bar
elsif ((vloc >= 490) and notbelowbasebar and (hloc >= 110) and (hloc < 690)) then
	pixelData <= "11111111";
else null;
end if;

--turn the column green if a corresponding correct hit is made
if firstbar and hitrange then 
	 if hit0 = "11" then
	    pixelData <= "00001100";
	 else null;
	 end if;
elsif secondbar and hitrange then
	if hit1 = "11" then
	    pixelData <= "00001100";
	 else null;
	 end if;
elsif thirdbar and hitrange then
	 if hit2 = "11" then
	    pixelData <= "00001100";
	 else null;
    end if;
elsif forthbar and hitrange then
	 if hit3 = "11" then
	    pixelData <= "00001100";
	 else null;
    end if;
else null;
end if;

--decoding 120-bits and dump them onto the screen
for i in 29 downto 0 loop
widthofbar := vloc >= newvv and vloc <= newvv + 10;
j := i * 4;       
      if wholelist(j + 3) = '1' and forthbar and widthofbar and notbelowbasebar then
            pixelData <= shine;
		else null;
		end if;

		if wholelist(j + 2) = '1' and thirdbar and widthofbar and notbelowbasebar then
            pixelData <= shine;
      else null;
      end if;
		
		if wholelist(j + 1) = '1' and secondbar and widthofbar and notbelowbasebar then
            pixelData <= shine;
      else null;
      end if;

		if wholelist(j) = '1' and firstbar and widthofbar and notbelowbasebar then
            pixelData <= shine;
      else null;
		end if;

newvv := newvv + 10; 
end loop;

--if grade is calculated, print it out
if sdone = '1' then
    pixelData <= "00000000";
   case grade is
     --F
       when "10" => if (vloc >= 150 and vloc <= 270) and (hloc >= 330 and hloc <= 370) then
                      pixelData <= "00000011";
                    else null;
						  end if;
                   if ((vloc >= 150 and vloc <= 170) or (vloc >= 200 and vloc <= 220)) and (hloc >= 370 and hloc <= 470) then  
                      pixelData <= "00000011";
						 else null;
                   end if;
	  --A
	  when "00" => if vloc >= 150 and vloc <= 170 and hloc >= 350 and hloc <= 450 then  
	                pixelData <= "00001100";
						  else null;
				       end if;
                   if (vloc >= 200 and vloc <= 220) and (hloc >= 370 and hloc <= 430) then
                    pixelData <= "00001100";
						   else null;
                   end if;
                   if (vloc >= 170 and vloc <= 270) and ((hloc >= 330 and hloc <= 370) or (hloc >= 430 and hloc <= 470)) then
                    pixelData <= "00001100";
						   else null;
                   end if;
     --B
      when "01" => if (vloc >= 150 and vloc <= 270) and (hloc >= 330 and hloc <= 370) then
                     pixelData <= "00110000";
							 else null;
                   end if;
                   if ((vloc >= 150 and vloc <= 200) or (vloc >= 220 and vloc <= 270)) and (hloc >= 440 and hloc <= 470) then
                     pixelData <= "00110000";
							 else null;
                   end if;
                    if ((vloc >= 150 and vloc <= 170) or (vloc >= 200 and vloc <= 220) or (vloc >= 250 and vloc <= 270) ) and (hloc >= 370 and hloc <= 440) then
                      pixelData <= "00110000";
							  else null;
                   end if;
	  when others => null;
	 end case;
else
   null;
end if;

end process;
-------------------------------------------------process---------------------------------------------------------
pixel <= pixelData;

-- only start VGA after RAMDAC has been programmed
resetVGA <= rstn AND startVGA;

-- pass the outputs out
hsync <= hsyncINT;
vsync <= vsyncINT;

-- Provide 50MHz pixel clock
pixelclk <= clk;

-- turn off the ethernet outputs and the right SRAM bank to avoid contention on the lines
triste <= '1';
rramce <= '1';
end vga_arch;
